{% extends 'buybackprogram/page.html' %}
{% load i18n %}
{% load humanize %}
{% load bootstrap %}
{% load static %}

{% block body %}
<div class="panel-heading">
  <h3 class="panel-title">Program Special Taxes</h3>
</div>
<div class="panel-body" style="min-height: 100px;">
  {% if perms.buybackprogram.manage_programs and program.owner.user == request.user %}
  <div>
    <form class="form" action="{% url 'buybackprogram:program_edit_item' program_pk=program.id %}" method="POST">
      {% csrf_token %}
      {{ form|bootstrap }}
      <button type="submit" class="btn btn-primary">Add item</button>
    </form>
    <a class="btn btn-default navbar-btn btn-warning" href="{% url 'buybackprogram:program_special_taxes' program_pk=program.id %}"><i class="fas fa-long-arrow-alt-left">Item taxes</i></a>
  </div>
  {% endif %}

</div>
{% endblock %}

{% block extra_javascript %}
<script type="text/javascript" src="{% static 'buybackprogram/js/bootstrap-autocomplete.min.js' %}"></script>
<script type="text/javascript" src="{% static 'buybackprogram/js/autocomplete.js' %}"></script>
{% endblock %}

{% block extra_css %}
{% endblock %}

{% block extra_script %}
{% endblock %}
