{% extends 'buybackprogram/page.html' %}
{% load i18n %}
{% load humanize %}
{% load bootstrap %}
{% load static %}

{% block body %}
<div class="panel-heading">
  <h3 class="panel-title">Edit program</h3>
</div>
<div class="panel-body" style="min-height: 100px;">
  <form class="form" action="{% url 'buybackprogram:program_edit' program_pk=program.id %}" method="POST">
    {% csrf_token %}
    {{ form|bootstrap }}
    <button type="submit" class="btn btn-primary">Edit Program</button>
  </form>
  {% endblock %}


  {% block extra_javascript %}
  <script type="text/javascript" src="{% static 'buybackprogram/js/bootstrap-autocomplete.min.js' %}"></script>
  <script type="text/javascript" src="{% static 'buybackprogram/js/autocomplete.js' %}"></script>
</div>
{% endblock %}

{% block extra_css %}
{% endblock %}

{% block extra_script %}
{% endblock %}
